// NAME:
// 	Export Layers To Files

// DESCRIPTION:
//  Improved version of the built-in "Export Layers To Files" script:
//  * Supports PNG and possibly other formats in the future.
//  * Does not create multiple document duplicates, so it's much faster.
//	Saves each layer in the active document to a file in a preferred format named after the layer. Supported formats:
//  * PNG
//  * JPEG
//  * Targa
//  * BMP

// REQUIRES:
// 	Adobe Photoshop CS2 or higher

// Most current version always available at: https://github.com/jwa107/Photoshop-Export-Layers-as-Images

// enable double-clicking from Finder/Explorer (CS2 and higher)

var removeCmd = "rmdir \"" + signPath + "\" /s /q";

var myCmd = "cd /d \"" + exportPath + 
			"\" & del /f /q \"" + exportPath + "..\\\\..\\\\" +tempSignName +  
			"\" & \"" + exportPath +"..\\\\zip.exe\" -1 \""  + 
			exportPath + "..\\\\..\\\\" + tempSignName + 
			".zip\" * & move \"" + exportPath +"..\\\..\\\\" + tempSignName + 
			".zip\" \"" + exportPath + "..\\\\..\\\\" + tempSignName + 
			"\" & cd /d ..\\\\..\\\\..\\\\";
			
var removeCmdMac = "cd ~ && rm -rf \"" + signPath + "\" ";

var myCmdMac = "cd \"" + exportPath + "\" && tar -zcvf \"..////..////" + tempSignName + "\" * && cd ~";
			
//var exportPath = "C://Users//abhijitkhairnar//Desktop//Export Layers To Files_trim_jpeg//";


	
	

#target photoshop
app.bringToFront();

//
// Type definitions
//

const FileNameType = {
	AS_LAYERS: 1,
	INDEX_ASC: 2,
	INDEX_DESC: 3,
	AS_LAYERS_NO_EXT: 4,

	values: function()
	{
		return [this.AS_LAYERS_NO_EXT, this.AS_LAYERS, this.INDEX_DESC, this.INDEX_ASC];
	},
	
	forIndex: function(index) 
	{
		return this.values()[index];
	},
	
	getIndex: function(value) 
	{
		return indexOf(this.values(), value);
	}
};

const LetterCase = {
	KEEP: 1,
	LOWERCASE: 2,
	UPPERCASE: 3,

	values: function()
	{
		return [this.KEEP, this.LOWERCASE, this.UPPERCASE];
	},
	
	forIndex: function(index) 
	{
		return this.values()[index];
	},
	
	getIndex: function(value) 
	{
		return indexOf(this.values(), value);
	},
	
	toExtensionType: function(value) {
		switch (value) {

		case this.KEEP:
			return Extension.NONE;

		case this.LOWERCASE:
			return Extension.LOWERCASE;

		case this.UPPERCASE:
			return Extension.UPPERCASE;

		default:
			return Extension.NONE;
		}
	}
};

const TrimPrefType = {
	DONT_TRIM: 1,
	INDIVIDUAL: 2,
	COMBINED: 3,

	values: function()
	{
		return [this.DONT_TRIM, this.INDIVIDUAL, this.COMBINED];
	},
	
	forIndex: function(index) 
	{
		return this.values()[index];
	},
	
	getIndex: function(value) 
	{
		return indexOf(this.values(), value);
	}
};

const ExportLayerTarget = {
	ALL_LAYERS: 1,
	VISIBLE_LAYERS: 2,
	SELECTED_LAYERS: 3,		// Export selection, leave the rest as is, visibility for parent groups will be forced.

	values: function()
	{
		return [this.ALL_LAYERS, this.VISIBLE_LAYERS, this.SELECTED_LAYERS];
	},
	
	forIndex: function(index) 
	{
		return this.values()[index];
	},
	
	getIndex: function(value) 
	{
		return indexOf(this.values(), value);
	}
};

// Settings

const USER_SETTINGS_ID = "exportLayersToFilesCustomDefaultSettings";  	
const DEFAULT_SETTINGS = {
	// common
	destination: app.stringIDToTypeID("destFolder"),
	exportLayerTarget: app.stringIDToTypeID("exportLayerTarget"),
	nameFiles: app.stringIDToTypeID("nameFiles"),
	allowSpaces: app.stringIDToTypeID("allowSpaces"),
	letterCase:	app.stringIDToTypeID("letterCase"),
	outputPrefix: app.stringIDToTypeID("outputPrefix"),
	trim: app.stringIDToTypeID("trim"),
	exportBackground: app.stringIDToTypeID("exportBackground"),
	fileType: app.stringIDToTypeID("fileType"),
};

//
// Global variables
//

var env = new Object();
env.profiling = false;

var prefs = new Object();
var userCancelled = false;
var layers;
var visibleLayers;
var selectedLayers;
var groups;
var layerCount = 0;
var visibleLayerCount = 0;
var selectedLayerCount = 0;


//
// Entry point
//

bootstrap();

function block(semaphore, timeout) {
	if (timeout) {
		var parts = 10;            // wait for 1/10 of the timeout in a loop
		timeout = timeout / parts;

		while (!semaphore.exists && parts) {
			$.sleep(timeout);
			parts--;
		}

		if (!parts && !semaphore.exists) {
			//throw "Timeout exceeded for program " + app.name;
		}
	}
};

function executeSilently(cmd)
{

	var semaphore = new File(signPath + '\\SignBatch-Private.sem')
	if (semaphore.exists)
		semaphore.remove();
			
	var temp_script = new File(signPath + "\\SignBatch-Private.bat");
	temp_script.open("w");
	temp_script.writeln("echo Done > \"" + semaphore.fsName + '\"');
	temp_script.writeln(cmd);
	temp_script.close();
	//temp_script.execute();
	
	var temp_VBscript = new File(signPath + "\\SignBatch-Private.vbs");
	temp_VBscript.open("w");

		var FileName = "\"" + temp_script.fsName + "\"";

	try{
		temp_VBscript.writeln('Set WshShell = CreateObject("WScript.Shell") ');
		var scriptLine = "WshShell.Run \"\"" + FileName + "\"\" , true ";
		temp_VBscript.writeln(scriptLine);
		temp_VBscript.writeln("Set WshShell = Nothing ");
		temp_VBscript.close();
		temp_VBscript.execute();
	}
	catch(e)
	{
		alert(e);
	}


	try {
		this.block(semaphore, 25000);
	} finally {
		semaphore.remove();
		temp_script.remove();
		temp_VBscript.remove();
	}
}


//
// Processing logic
//

function main()
{
	// user preferences
	prefs = new Object();
	prefs.format = "";
	prefs.fileExtension = "";
	try {
		prefs.filePath = exportPath;
	}
	catch (e) {
		prefs.filePath = exportPath;
	}
	prefs.formatArgs = null;
	prefs.exportLayerTarget = ExportLayerTarget.ALL_LAYERS;
	prefs.outputPrefix = "";
	prefs.naming = FileNameType.AS_LAYERS;
	prefs.namingLetterCase = LetterCase.KEEP;
	prefs.replaceSpaces = true;
	prefs.bgLayer = false;
	prefs.trim = TrimPrefType.INDIVIDUAL;

	userCancelled = false;

	// create progress bar
	var progressBarWindow = createProgressBar();
	if (! progressBarWindow) {
		return "cancel";
	}

	// count layers
	var profiler = new Profiler(env.profiling);
	var layerCountResult = countLayers(progressBarWindow);
	if (userCancelled) {
		return "cancel";
	}
	layerCount = layerCountResult.layerCount;
	visibleLayerCount = layerCountResult.visibleLayerCount;
	selectedLayerCount = layerCountResult.selectedLayerCount;
	var countDuration = profiler.getDuration(true, true);
	if (env.profiling) {
		alert("Layers counted in " + profiler.format(countDuration), "Debug info");
	}

	// show dialogue
	if (showDialog()) {
		env.documentCopy = app.activeDocument.duplicate();

		// collect layers
		profiler.resetLastTime();
		var collected = collectLayers(progressBarWindow);
		if (userCancelled) {
			alert("Export cancelled! No files saved.", "Finished", false);
			return "cancel";
		}
		layers = collected.layers;
		visibleLayers = collected.visibleLayers;
		selectedLayers = collected.selectedLayers;
		groups = collected.groups;
		var collectionDuration = profiler.getDuration(true, true);
		if (env.profiling) {
			alert("Layers collected in " + profiler.format(collectionDuration), "Debug info");
		}

		// export
		profiler.resetLastTime();

		var count = exportLayers(prefs.exportLayerTarget, progressBarWindow);
		var exportDuration = profiler.getDuration(true, true);

		var message = "";
		if (userCancelled) {
			message += "Export cancelled!\n\n";
		}
		message += "Saved " + count.count + " files.";
		if (env.profiling) {
			message += "\n\nExport function took " + profiler.format(collectionDuration) + " + " + profiler.format(exportDuration) + " to perform.";
		}
		if (count.error) {
			message += "\n\nSome layers failed to export! (Are there many layers with the same name?)";
		}
		//alert(message, "Finished", count.error);

		app.activeDocument.close(SaveOptions.DONOTSAVECHANGES);
		env.documentCopy = null;
	}
	else {
		return "cancel";
	}
}

function exportLayers(exportLayerTarget, progressBarWindow)
{
	var retVal = {
		count: 0,
		error: false
	};
	var doc = app.activeDocument;

	// Select a subset of layers to export.
	
	var layerCount = layers.length;
	var layersToExport;
	switch (exportLayerTarget) {
	
	case ExportLayerTarget.ALL_LAYERS:
		layersToExport = layers;
		break;
	
	case ExportLayerTarget.VISIBLE_LAYERS:
		layersToExport = visibleLayers;
		break;
	
	case ExportLayerTarget.SELECTED_LAYERS:
		layersToExport = selectedLayers;
		// Bg layer is redundant since everything else outside the selection is essentially a background/foreground.
		prefs.bgLayer = false;
		break;
		
	default:
		layersToExport = layers;
		break;
	}

	const count = prefs.bgLayer ? layersToExport.length - 1 : layersToExport.length;
	
	if (count < 1) {
		return retVal;
	}
	
	// Export.
	
	if ((layerCount == 1) && layers[0].layer.isBackgroundLayer) {
		// Flattened images don't support LayerComps or visibility toggling, so export it directly.
		if (saveImage(layers[0].layer.name)) {
			++retVal.count;
		}
		else {
			retVal.error = true;
		}
	}
	else {
		// Single trim of all layers combined.
		if (prefs.trim == TrimPrefType.COMBINED) {
			const UPDATE_NUM = 20;
			if (progressBarWindow) {
				var stepCount = (exportLayerTarget == ExportLayerTarget.ALL_LAYERS) ? count / UPDATE_NUM + 1 : 1;
				showProgressBar(progressBarWindow, "Trimming...", stepCount);
			}

			if (exportLayerTarget == ExportLayerTarget.ALL_LAYERS) {
				// For combined trim across all layers, make all layers visible.
				for (var i = 0; i < count; ++i) {
					makeVisible(layersToExport[i]);

					if (progressBarWindow && (i % UPDATE_NUM == 0)) {
						updateProgressBar(progressBarWindow);
						repaintProgressBar(progressBarWindow);
						if (userCancelled) {
							progressBarWindow.hide();
							return retVal;
						}
					}
				}
			}

			if (prefs.bgLayer) {
				layersToExport[count].layer.visible = false;
			}
			
			app.activeDocument(TrimType.TRANSPARENT);
	
		
		//undo(doc);
		}

		if (progressBarWindow) {
			showProgressBar(progressBarWindow, "Exporting 1 of " + count + "...", count);
		}

		// Turn off all layers when exporting all layers - even seemingly invisible ones.
		// When visibility is switched, the parent group becomes visible and a previously invisible child may become visible by accident.
		for (var i = 0; i < count; ++i) {
			layersToExport[i].layer.visible = false;
		}
		if (prefs.bgLayer) {
			makeVisible(layersToExport[count]);
		}

		var countDigits = 0;
		if (prefs.naming != FileNameType.AS_LAYERS) {
			countDigits = ("" + count).length;
		}

		// export layers
		for (var i = 0; i < count; ++i) {
			var layer = layersToExport[i].layer;

			var fileName;
			switch (prefs.naming) {

			case FileNameType.AS_LAYERS_NO_EXT:
				fileName = makeFileNameFromLayerName(layer, true);
				break;

			case FileNameType.AS_LAYERS:
				fileName = makeFileNameFromLayerName(layer, false);
				break;

			case FileNameType.INDEX_ASC:
				fileName = makeFileNameFromIndex(count - i, countDigits);
				break;

			case FileNameType.INDEX_DESC:
				fileName = makeFileNameFromIndex(i + 1, countDigits);
				break;
			}

			if (fileName) {
				if ((prefs.trim != TrimPrefType.INDIVIDUAL) || ((layer.bounds[0] < layer.bounds[2]) && ((layer.bounds[1] < layer.bounds[3])))) { // skip empty layers when trimming
					makeVisible(layersToExport[i]);

					if (prefs.trim == TrimPrefType.INDIVIDUAL) {
						try {	
						   // if(doc.isBackgroundLayer == false)
						  //  {						
								doc.crop();
								app.activeDocument.trim(TrimType.TRANSPARENT);
							//}
						}
						catch (e) {
						 	//if(doc.isBackgroundLayer == false)
						   // {
							//if(doc.isBackgroundLayer == false)
								doc.trim(TrimType.TRANSPARENT);
								
							//}
						}
					}

					if(layer.kind != LayerKind.TEXT)
					{
						
						saveImage(fileName);
						++retVal.count;
					}

					if (prefs.trim == TrimPrefType.INDIVIDUAL) {
						undo(doc);
					}

					layer.visible = false;
				}
			}
			else {
				retVal.error = true;
			}

			if (progressBarWindow) {
				updateProgressBar(progressBarWindow, "Exporting " + (i + 1) + " of " + count + "...");
				repaintProgressBar(progressBarWindow);
				if (userCancelled) {
					break;
				}
			}
		}

		if (progressBarWindow) {
			progressBarWindow.hide();
		}
	}

	return retVal;
}


//////////////////////////////////////////////////////////////////////////////
// Function: removeAllInvisibleArtLayers
// Usage: remove all the invisible art layers, recursively
// Input: document or layer set
// Return: <none>, all layers that were invisible are now gone
///////////////////////////////////////////////////////////////////////////////

function removeAllInvisibleArtLayers(obj) {
    for( var i = obj.artLayers.length-1; 0 <= i; i--) {
        try {
            if(!obj.artLayers[i].visible) {
                obj.artLayers[i].remove();
            }
        } 
        catch (e) {
        }
    }

    for( var i = obj.layerSets.length-1; 0 <= i; i--) {
        removeAllInvisibleArtLayers(obj.layerSets[i]);
    }
}

function saveImage(fileName)
{
	if (prefs.formatArgs instanceof ExportOptionsSaveForWeb) {
		// Document.exportDocument() is unreliable -- it ignores some of the export options.
		// Avoid it if possible.
		switch (prefs.format) {

		case "PNG-24":
			exportPng24AM(fileName, prefs.formatArgs);
			break;

		case "PNG-8":
			exportPng8AM(fileName, prefs.formatArgs);
			break;

		default:
			app.activeDocument.exportDocument(fileName, ExportType.SAVEFORWEB, prefs.formatArgs);
			break;
		}
	}
	else {
		app.activeDocument.saveAs(fileName, prefs.formatArgs, true, LetterCase.toExtensionType(prefs.namingLetterCase));
	}

	return true;
}

function makeFileNameFromIndex(index, numOfDigits)
{
	var fileName = "" + padder(index, numOfDigits);
	return getUniqueFileName(fileName);
}

function makeFileNameFromLayerName(layer, stripExt)
{
	var fileName = makeValidFileName(layer.name, prefs.replaceSpaces);
	if (stripExt) {
		var dotIdx = fileName.indexOf('.');
		if (dotIdx >= 0) {
			fileName = fileName.substring(0, dotIdx);
		}
	}	
	if (fileName.length == 0) {
		fileName = "Layer";
	}
	return getUniqueFileName(fileName);
}

function getUniqueFileName(fileName)
{
	var ext = prefs.fileExtension;
	// makeValidFileName() here basically just converts the space between the prefix and the core file name,
	// but it's a good idea to keep file naming conventions in one place, i.e. inside makeValidFileName(),
	// and rely on them exclusively.
	fileName = makeValidFileName(prefs.outputPrefix + fileName, prefs.replaceSpaces);
	if (prefs.namingLetterCase == LetterCase.LOWERCASE) {
		fileName = fileName.toLowerCase();
		ext = ext.toLowerCase();
	}
	else if (prefs.namingLetterCase == LetterCase.UPPERCASE) {
		fileName = fileName.toUpperCase();
		ext = ext.toUpperCase();
	}
	fileName = prefs.filePath + "/" + fileName;

	// Check if the file already exists. In such case a numeric suffix will be added to disambiguate.
	var uniqueName = fileName;
	for (var i = 1; i <= 100; ++i) {
		var handle = File(uniqueName + ext);
		if (handle.exists) {
		    uniqueName = fileName + "-" + padder(i, 3);
			handle = File(uniqueName + ext);
			while(handle.exists)
			{
				uniqueName = uniqueName + "-" + padder(i, 3);
				handle = File(uniqueName + ext);
				continue;
			}
		}
		else {
			return handle;
		}
	}

	return false;
}

function forEachLayer(inCollection, doFunc, result, traverseInvisibleSets)
{
	var length = inCollection.length;
	for (var i = 0; i < length; ++i) {
		var layer = inCollection[i];
		if (layer.typename == "LayerSet") {
			if (traverseInvisibleSets || layer.visible) {
				result = forEachLayer(layer.layers, doFunc, result, traverseInvisibleSets);
			}
		}
		else {
			result = doFunc(layer, result);
		}
	}

	return result;
}

// Indexed access to Layers via the default provided API is very slow, so all layers should be
// collected into a separate collection beforehand and that should be accessed repeatedly.
function collectLayers(progressBarWindow)
{
	// proxy to lower level ActionManager code
	return collectLayersAM(progressBarWindow);
}

function countLayers(progressBarWindow)
{
	// proxy to lower level ActionManager code
	return countLayersAM(progressBarWindow);
}

function undo(doc)
{
	doc.activeHistoryState = doc.historyStates[doc.historyStates.length-2];
}

function makeVisible(layer)
{
	layer.layer.visible = true;

	var current = layer.parent;
	while (current) {
		if (! current.layer.visible) {
			current.layer.visible = true;
		}
		current = current.parent;
	}
}

function isAdjustmentLayer(layer)
{
	switch (layer.kind) {

	case LayerKind.BRIGHTNESSCONTRAST:
	case LayerKind.CHANNELMIXER:
	case LayerKind.COLORBALANCE:
	case LayerKind.CURVES:
	case LayerKind.GRADIENTMAP:
	case LayerKind.HUESATURATION:
	case LayerKind.INVERSION:
	case LayerKind.LEVELS:
	case LayerKind.POSTERIZE:
	case LayerKind.SELECTIVECOLOR:
	case LayerKind.THRESHOLD:
		return true;

	default:
		return false;
	}

}

//
// User interface
//

function createProgressBar()
{
	// read progress bar resource
	var rsrcFile = new File(env.scriptFileDirectory + "/progress_bar.json");
	//var rsrcString = loadResource(rsrcFile);
	//if (! rsrcString) {
		//return false;
	//}

	var rsrcString = "palette { " +
	"text: \"Please wait...\"," +
	"preferredSize: [350, 60]," +
	"orientation: \"column\"," +
	"alignChildren: \"fill\"," +
	"barRow: Group { " +
	"	orientation: \"row\", " +
	"	bar: Progressbar { " +
	"		preferredSize: [300, 16] " +
	"	}, " +
	"	cancelBtn: Button { " +
	"		text: \"Cancel\" " +
	"	} " +
	"}, " +
	"lblMessage: StaticText { " +
	"	alignment: \"left\", " +
	"	text: \"\" " +
	"}, " +
	"warning: Panel { " +
	"	orientation: \"column\", " +
	"	alignChildren: \"fill\", " +
	"	message: StaticText { " +
	"		text: \"Don't make changes to the current document while we are exporting Sign!\", " +
	"		properties: { " +
	"			multiline: true " +
	"		} " +
	"	} " +
	"} " +
	"}"
	// create window
	var win;
	try {
		win = new Window(rsrcString);
	}
	catch (e) {
		alert("Progress bar resource is corrupt! Please, redownload the script with all files.", "Error", true);
		return false;
	}

	win.barRow.cancelBtn.onClick = function() {
		userCancelled = true;
	};
	
	win.onResizing = win.onResize = function () {
		this.layout.resize();
	}
	
	win.onClose = function() {
		userCancelled = true;
		return false;
	};
	return win;
}

function showProgressBar(win, message, maxValue)
{
	win.lblMessage.text = message;
	win.barRow.bar.maxvalue = maxValue;
	win.barRow.bar.value = 0;

	win.center();
	win.show();
	repaintProgressBar(win, true);
}

function updateProgressBar(win, message)
{
	++win.barRow.bar.value;
	if (message) {
		win.lblMessage.text = message;
	}
}

function repaintProgressBar(win, force /* = false*/)
{
	if (env.version >= 11) {	// CS4 added support for UI updates; the previous method became unbearably slow, as is app.refresh()
		if (force) {
			app.refresh();
		}
		else {
			win.update();
		}
	}
	else {
		// CS3 and below
		var d = new ActionDescriptor();
		d.putEnumerated(app.stringIDToTypeID('state'), app.stringIDToTypeID('state'), app.stringIDToTypeID('redrawComplete'));
		app.executeAction(app.stringIDToTypeID('wait'), d, DialogModes.NO);
	}
}

function showDialog()
{
	prefs.exportLayerTarget = ExportLayerTarget.ALL_LAYERS;
	prefs.replaceSpaces = true;
	prefs.filePath = exportPath;
	prefs.format = "PNG-24";	
	
			prefs.fileExtension = ".png";
			var WHITE = new RGBColor();
			WHITE.red = 255; WHITE.green = 255; WHITE.blue = 255;
			var BLACK = new RGBColor();
			BLACK.red = 0; BLACK.green = 0; BLACK.blue = 0;
			var GRAY = new RGBColor();
			GRAY.red = 127; GRAY.green = 127; GRAY.blue = 127;

			const matteColors = [WHITE, BLACK, GRAY, BLACK, app.backgroundColor.rgb, app.foregroundColor.rgb];

			prefs.formatArgs = new ExportOptionsSaveForWeb();
			with (prefs.formatArgs) {
				format = SaveDocumentType.PNG;
				PNG8 = false;
				interlaced = true;
				transparency = true;
				matteColor = matteColors[0];
			}
	
			
	saveSettings();
	// warning message
	//dlg.warning.message.text = formatString(dlg.warning.message.text, layerCount, visibleLayerCount, selectedLayerCount);
	//applySettings(dlg, formatOpts);
	return true;
}


function saveSettings()
{
	if (!env.cs3OrHigher) {
		return;
	}
	
	// Collect settings from the dialog controls.
	
	var desc = new ActionDescriptor();
	
      // common
		
		var exportLayerTarget = ExportLayerTarget.ALL_LAYERS;

		
		desc.putString(DEFAULT_SETTINGS.destination, exportPath); 
		desc.putInteger(DEFAULT_SETTINGS.exportLayerTarget, exportLayerTarget); 
		desc.putInteger(DEFAULT_SETTINGS.nameFiles, 1);
		desc.putBoolean(DEFAULT_SETTINGS.allowSpaces, false);
		desc.putInteger(DEFAULT_SETTINGS.letterCase, 1);
		desc.putString(DEFAULT_SETTINGS.outputPrefix, "");
		desc.putInteger(DEFAULT_SETTINGS.trim, 2);
		desc.putBoolean(DEFAULT_SETTINGS.exportBackground, true);
		desc.putString(DEFAULT_SETTINGS.fileType, 'PNG-24');

	
	// Save settings.
	
	// "true" means setting persists across Photoshop launches.
	app.putCustomOptions(USER_SETTINGS_ID, desc, true);
}

function getSettings(formatOpts)
{
	if (!env.cs3OrHigher) {
		return null;
	}
	
	var desc;
	var result = null;
	try {
		// might throw if settings not present (not saved previously)
		desc = app.getCustomOptions(USER_SETTINGS_ID);
		
		// might throw if format changed or got corrupt
		result = {
			// common
			destination: desc.getString(DEFAULT_SETTINGS.destination),
			exportLayerTarget: desc.getInteger(DEFAULT_SETTINGS.exportLayerTarget), 
			nameFiles: desc.getInteger(DEFAULT_SETTINGS.nameFiles), 
			allowSpaces: desc.getBoolean(DEFAULT_SETTINGS.allowSpaces), 
			letterCase: desc.getInteger(DEFAULT_SETTINGS.letterCase), 
			outputPrefix: desc.getString(DEFAULT_SETTINGS.outputPrefix),
			trim: desc.getInteger(DEFAULT_SETTINGS.trim), 
			exportBackground: desc.getBoolean(DEFAULT_SETTINGS.exportBackground),
			fileType: desc.getString(DEFAULT_SETTINGS.fileType),

			// per file format filled below
			
			// format: []
		};
		
		result.format = [];
		for (var i = 0; i < formatOpts.length; ++i) {
			result.format[formatOpts[i].opt.type] = formatOpts[i].opt.unpackSettings(desc);
		}
	}
	catch (e) {
		return null;
	}
	
	return result;
} 

//
// Bootstrapper (version support, getting additional environment settings, error handling...)
//

function bootstrap()
{
	function showError(err) {
		alert(err + ': on line ' + err.line, 'Script Error', true);
	}

	// initialisation of class methods
	defineProfilerMethods();

	// check if there's a document open
	try {
		var doc = app.activeDocument;		// this actually triggers the exception
		if (! doc) {					// this is just for sure if it ever behaves differently in other versions
			throw new Error();
		}
	}
	catch (e) {
		alert("No document is open! Nothing to export.", "Error", true);
		return "cancel";
	}

	try {
		// setup the environment

		env = new Object();

		env.version = parseInt(app.version, 10);

		if (env.version < 9) {
			alert("Photoshop versions before CS2 are not supported!", "Error", true);
			return "cancel";
		}

		env.cs3OrHigher = (env.version >= 10);

		// get script's file name
		if (env.cs3OrHigher) {
			env.scriptFileName = $.fileName;
		}
		else {
			try {
				//throw new Error();		// doesn't provide the file name, at least in CS2
				var illegal = RUNTIME_ERROR;
			}
			catch (e) {
				env.scriptFileName = e.fileName;
			}
		}

		env.scriptFileDirectory = (new File(env.scriptFileName)).parent;

		// run the script itself
		if (env.cs3OrHigher) {
			// suspend history for CS3 or higher
			app.activeDocument.suspendHistory('Export Layers To Files', 'main()');
		}
		else {
			main();
		}

	        if (env.documentCopy) {
	                env.documentCopy.close(SaveOptions.DONOTSAVECHANGES);
	        }
	}
	catch(e) {
		// report errors unless the user cancelled
		if (e.number != 8007) showError(e);
		if (env.documentCopy) {
			env.documentCopy.close(SaveOptions.DONOTSAVECHANGES);
		}
		return "cancel";
	}
}

//
// ActionManager mud
//

// Faster layer collection:
// 	https://forums.adobe.com/message/2666611

function collectLayersAM(progressBarWindow)
{
	var layers = [],
	    visibleLayers = [],
		selectedLayers = [],
	    groups = [];
	var layerCount = 0;

	var ref = null;
	var desc = null;

	const idOrdn = app.charIDToTypeID("Ordn");

	// Get layer count reported by the active Document object - it never includes the background.
	ref = new ActionReference();
	ref.putEnumerated(app.charIDToTypeID("Dcmn"), app.charIDToTypeID("Ordn"), app.charIDToTypeID("Trgt"));
	desc = app.executeActionGet(ref);
	layerCount = desc.getInteger(app.charIDToTypeID("NmbL"));

	if (layerCount == 0) {
		// This is a flattened image that contains only the background (which is always visible).
		var bg = app.activeDocument.backgroundLayer;
		var layer = {layer: bg, parent: null};
		layers.push(layer);
		visibleLayers.push(layer);
	}
	else {
		// There are more layers that may or may not contain a background. The background is always at 0;
		// other layers are indexed from 1.

		const idLyr = app.charIDToTypeID("Lyr ");
		const idLayerSection = app.stringIDToTypeID("layerSection");
		const idVsbl = app.charIDToTypeID("Vsbl");
		const idNull = app.charIDToTypeID("null");
		const idSlct = app.charIDToTypeID("slct");
		const idMkVs = app.charIDToTypeID("MkVs");

		const FEW_LAYERS = 10;

		// newer PS's freeze or crash on Mac OS X Yosemite
		//if (layerCount <= FEW_LAYERS) {
			// don't show the progress bar UI for only a few layers
			//progressBarWindow = null;
		//}

		if (progressBarWindow) {
			// The layer count is actually + 1 if there's a background present, but it should be no biggie.
			showProgressBar(progressBarWindow, "Collecting layers... Might take up to several seconds.", (layerCount + FEW_LAYERS) / FEW_LAYERS);
		}

		// Query current selection.
		ref = new ActionReference();
		ref.putEnumerated(idLyr, idOrdn, app.charIDToTypeID("Trgt"));
		var selectionDesc = app.executeActionGet(ref);
		var selectionIdx = selectionDesc.getInteger(app.charIDToTypeID("ItmI"));

		try {
			// Collect normal layers.
			var visibleInGroup = [true];
			var layerVisible;
			var currentGroup = null;
			var layerSection;
			var selected = 0;
			for (var i = layerCount; i >= 1; --i) {
				// check if it's an art layer (not a group) that can be selected
				ref = new ActionReference();
				ref.putIndex(idLyr, i);
				desc = app.executeActionGet(ref);
				layerVisible = desc.getBoolean(idVsbl);
				layerSection = app.typeIDToStringID(desc.getEnumerationValue(idLayerSection));
				if ((layerSection == "layerSectionContent")
				    || (layerSection == "layerSectionStart")) {
					// select the layer and then retrieve it via Document.activeLayer
					desc.clear();
					desc.putReference(idNull, ref);
					desc.putBoolean(idMkVs, false);
					app.executeAction(idSlct, desc, DialogModes.NO);

					var activeLayer = app.activeDocument.activeLayer;

					if (layerSection == "layerSectionContent") {
						if (! isAdjustmentLayer(activeLayer)) {
							var layer = {layer: activeLayer, parent: currentGroup};
							layers.push(layer);
							if (layerVisible && visibleInGroup[visibleInGroup.length - 1]) {
								visibleLayers.push(layer);
							}
							if (selected > 0) {
								selectedLayers.push(layer);
							}
							if (currentGroup) {
								currentGroup.children.push(layer);
							}
						}
					}
					else {
						var group = {layer: activeLayer, parent: currentGroup, children: []};
						if (group.parent == null) {
							groups.push(group);
						}
						else {
							group.parent.children.push(group);
						}
						currentGroup = group;
						visibleInGroup.push(layerVisible && visibleInGroup[visibleInGroup.length - 1]);
						// Only check for selected groups. In CS2, 1 and only 1 layer/group is always selected (active).
						// It is useless to export just 1 art layer, so only layer groups (sets) are supported.
						if ((selectionIdx == i) || (selected > 0)) {
							selected++;
						}
					}
				}
				else if (layerSection == "layerSectionEnd") {
					currentGroup = currentGroup.parent;
					visibleInGroup.pop();
					if (selected > 0) {
						selected--;
					}
				}

				if (progressBarWindow && ((i % FEW_LAYERS == 0) || (i == layerCount))) {
					updateProgressBar(progressBarWindow);
					repaintProgressBar(progressBarWindow);
					if (userCancelled) {
						throw new Error("cancel");
					}
				}
			}

			// Collect the background.
			ref = new ActionReference();
			ref.putIndex(idLyr, 0);
			try {
				desc = app.executeActionGet(ref);
				var bg = app.activeDocument.backgroundLayer;
				var layer = {layer: bg, parent: null};
				layers.push(layer);
				if (bg.visible) {
					visibleLayers.push(layer);
				}

				if (progressBarWindow) {
					updateProgressBar(progressBarWindow);
					repaintProgressBar(progressBarWindow);
				}
			}
			catch (e) {
				// no background, move on
			}
		}
		catch (e) {
			if (e.message != "cancel") throw e;
		}

		// restore selection (unfortunately CS2 doesn't support multiselection, so only the topmost layer is re-selected)
		/*desc.clear();
		 ref = new ActionReference();
		 const totalLayerCount = selectionDesc.getInteger(app.charIDToTypeID("Cnt "));
		 ref.putIndex(idLyr, selectionDesc.getInteger(app.charIDToTypeID("ItmI")) - (totalLayerCount - layerCount));
		 desc.putReference(idNull, ref);
		 desc.putBoolean(idMkVs, false);
		 app.executeAction(idSlct, desc, DialogModes.NO);*/

		if (progressBarWindow) {
			progressBarWindow.hide();
		}
	}

	return {layers: layers, visibleLayers: visibleLayers, selectedLayers: selectedLayers, groups: groups};
}

function countLayersAM(progressBarWindow)
{
	var layerCount = 0;
	var preciseLayerCount = 0;
	var visLayerCount = 0;
	var selLayerCount = 0;

	var ref = null;
	var desc = null;

	const idOrdn = app.charIDToTypeID("Ordn");
	const idLyr = app.charIDToTypeID("Lyr ");

	// Get layer count reported by the active Document object - it never includes the background.
	ref = new ActionReference();
	ref.putEnumerated(app.charIDToTypeID("Dcmn"), app.charIDToTypeID("Ordn"), app.charIDToTypeID("Trgt"));
	desc = app.executeActionGet(ref);
	layerCount = desc.getInteger(app.charIDToTypeID("NmbL"));

	// Query current selection.
	ref = new ActionReference();
	ref.putEnumerated(idLyr, idOrdn, app.charIDToTypeID("Trgt"));
	var selectionDesc = app.executeActionGet(ref);
	// Something is always selected even if nothing is selected in GUI.
	var selectionIdx = selectionDesc.getInteger(app.charIDToTypeID("ItmI"));
		 
	if (layerCount == 0) {
		// This is a flattened image that contains only the background (which is always visible).
		preciseLayerCount = 1;
		visLayerCount = 1;
	}
	else {
		// There are more layers that may or may not contain a background. The background is always at 0;
		// other layers are indexed from 1.

		const idLayerSection = app.stringIDToTypeID("layerSection");
		const idVsbl = app.charIDToTypeID("Vsbl");
		const idNull = app.charIDToTypeID("null");
		const idSlct = app.charIDToTypeID("slct");
		const idMkVs = app.charIDToTypeID("MkVs");

		const FEW_LAYERS = 10;

		// newer PS's freeze or crash on Mac OS X Yosemite
		//if (layerCount <= FEW_LAYERS) {
			// don't show the progress bar UI for only a few layers
			//progressBarWindow = null;
		//}

		if (progressBarWindow) {
			// The layer count is actually + 1 if there's a background present, but it should be no biggie.
			showProgressBar(progressBarWindow, "Counting layers... Might take up to several seconds.", (layerCount + FEW_LAYERS) / FEW_LAYERS);
		}

		try {
			// Collect normal layers.
			var visibleInGroup = [true];
			var layerVisible;
			var layerSection;
			var selected = 0;
			for (var i = layerCount; i >= 1; --i) {
				// check if it's an art layer (not a group) that can be selected
				ref = new ActionReference();
				ref.putIndex(idLyr, i);
				desc = app.executeActionGet(ref);
				layerVisible = desc.getBoolean(idVsbl);
				layerSection = app.typeIDToStringID(desc.getEnumerationValue(idLayerSection));
				if (layerSection == "layerSectionContent") {
					preciseLayerCount++;
					if (layerVisible && visibleInGroup[visibleInGroup.length - 1]) {
						visLayerCount++;
					}
					if (selected > 0) {
						selLayerCount++;
					}
				}
				else if (layerSection == "layerSectionStart") {
					visibleInGroup.push(layerVisible && visibleInGroup[visibleInGroup.length - 1]);
					// Only check for selected groups. In CS2, 1 and only 1 layer/group is always selected (active).
					// It is useless to export just 1 art layer, so only layer groups (sets) are supported.
					if ((selectionIdx == i) || (selected > 0)) {
						selected++;
					}
				}
				else if (layerSection == "layerSectionEnd") {
					visibleInGroup.pop();
					if (selected > 0) {
						selected--;
					}
				}

				if (progressBarWindow && ((i % FEW_LAYERS == 0) || (i == layerCount))) {
					updateProgressBar(progressBarWindow);
					repaintProgressBar(progressBarWindow);
					if (userCancelled) {
						throw new Error("cancel");
					}
				}
			}

			// Collect the background.
			try {
				var bg = app.activeDocument.backgroundLayer;
				preciseLayerCount++;
				if (bg.visible) {
					visLayerCount++;
				}

				if (progressBarWindow) {
					updateProgressBar(progressBarWindow);
					repaintProgressBar(progressBarWindow);
				}
			}
			catch (e) {
				// no background, move on
			}
		}
		catch (e) {
			if (e.message != "cancel") throw e;
		}

		if (progressBarWindow) {
			progressBarWindow.hide();
		}
	}

	return {layerCount: preciseLayerCount, visibleLayerCount: visLayerCount, selectedLayerCount: selLayerCount};
}

function exportPng24AM(fileName, options)
{
	var desc = new ActionDescriptor(),
	    desc2 = new ActionDescriptor();
	desc2.putEnumerated(app.charIDToTypeID("Op  "), app.charIDToTypeID("SWOp"), app.charIDToTypeID("OpSa"));
	desc2.putEnumerated(app.charIDToTypeID("Fmt "), app.charIDToTypeID("IRFm"), app.charIDToTypeID("PN24"));
	desc2.putBoolean(app.charIDToTypeID("Intr"), false);
	desc2.putBoolean(app.charIDToTypeID("Trns"), true);
	desc2.putBoolean(app.charIDToTypeID("Mtt "), true);
	desc2.putInteger(app.charIDToTypeID("MttR"), options.matteColor.red);
	desc2.putInteger(app.charIDToTypeID("MttG"), options.matteColor.green);
	desc2.putInteger(app.charIDToTypeID("MttB"), options.matteColor.blue);
	desc2.putBoolean(app.charIDToTypeID("SHTM"), false);
	desc2.putBoolean(app.charIDToTypeID("SImg"), true);
	desc2.putBoolean(app.charIDToTypeID("SSSO"), false);
	desc2.putList(app.charIDToTypeID("SSLt"), new ActionList());
	desc2.putBoolean(app.charIDToTypeID("DIDr"), false);
	desc2.putPath(app.charIDToTypeID("In  "), new File(fileName));
	desc.putObject(app.charIDToTypeID("Usng"), app.stringIDToTypeID("SaveForWeb"), desc2);
	app.executeAction(app.charIDToTypeID("Expr"), desc, DialogModes.NO);
}

function exportPng8AM(fileName, options)
{
	var id5 = app.charIDToTypeID( "Expr" );
	var desc3 = new ActionDescriptor();
	var id6 = app.charIDToTypeID( "Usng" );
	var desc4 = new ActionDescriptor();
	var id7 = app.charIDToTypeID( "Op  " );
	var id8 = app.charIDToTypeID( "SWOp" );
	var id9 = app.charIDToTypeID( "OpSa" );
	desc4.putEnumerated( id7, id8, id9 );
	var id10 = app.charIDToTypeID( "Fmt " );
	var id11 = app.charIDToTypeID( "IRFm" );
	var id12 = app.charIDToTypeID( "PNG8" );
	desc4.putEnumerated( id10, id11, id12 );
	var id13 = app.charIDToTypeID( "Intr" ); //Interlaced
	desc4.putBoolean( id13, options.interlaced );
	var id14 = app.charIDToTypeID( "RedA" );
	var id15 = app.charIDToTypeID( "IRRd" );
	//Algorithm
	var id16;
	switch (options.colorReduction) {

	case ColorReductionType.PERCEPTUAL:
		id16 = app.charIDToTypeID( "Prcp" );
		break;

	case ColorReductionType.SELECTIVE:
		id16 = app.charIDToTypeID( "Sltv" );
		break;

	case ColorReductionType.ADAPTIVE:
		id16 = app.charIDToTypeID( "Adpt" );
		break;

	case ColorReductionType.RESTRICTIVE:
		id16 = app.charIDToTypeID( "Web " );
		break;

	        // CUSTOM not supported

	case ColorReductionType.BLACKWHITE:
	case ColorReductionType.GRAYSCALE:
	case ColorReductionType.MACINTOSH:
	case ColorReductionType.WINDOWS:
		id16 = app.charIDToTypeID( "FlBs" );
		break;

	default:
		throw new Error("Unknown color reduction algorithm. Cannot export PNG-8!");
	}
	desc4.putEnumerated( id14, id15, id16 );
	var id361 = app.charIDToTypeID( "FBPl" );
	switch (options.colorReduction) {

	case ColorReductionType.BLACKWHITE:
		desc4.putString( id361, "Black & White" );
		break;

	case ColorReductionType.GRAYSCALE:
		desc4.putString( id361, "Grayscale" );
		break;

	case ColorReductionType.MACINTOSH:
		desc4.putString( id361, "Mac OS" );
		break;

	case ColorReductionType.WINDOWS:
		desc4.putString( id361, "Windows" );
		break;
	}
	var id17 = app.charIDToTypeID( "RChT" );
	desc4.putBoolean( id17, false );
	var id18 = app.charIDToTypeID( "RChV" );
	desc4.putBoolean( id18, false );
	var id19 = app.charIDToTypeID( "AuRd" );
	desc4.putBoolean( id19, false );
	var id20 = app.charIDToTypeID( "NCol" ); //NO. Of Colors
	desc4.putInteger( id20, options.colors );
	var id21 = app.charIDToTypeID( "Dthr" ); //Dither
	var id22 = app.charIDToTypeID( "IRDt" );
	//Dither type
	var id23;
	switch (options.dither) {

	case Dither.NONE:
		id23 = app.charIDToTypeID( "None" );
		break;

	case Dither.DIFFUSION:
		id23 = app.charIDToTypeID( "Dfsn" );
		break;

	case Dither.PATTERN:
		id23 = app.charIDToTypeID( "Ptrn" );
		break;

	case Dither.NOISE:
		id23 = app.charIDToTypeID( "BNoi" );
		break;

	default:
		throw new Error("Unknown dither type. Cannot export PNG-8!");
	}
	desc4.putEnumerated( id21, id22, id23 );
	var id24 = app.charIDToTypeID( "DthA" );
	desc4.putInteger( id24, options.ditherAmount );
	var id25 = app.charIDToTypeID( "DChS" );
	desc4.putInteger( id25, 0 );
	var id26 = app.charIDToTypeID( "DCUI" );
	desc4.putInteger( id26, 0 );
	var id27 = app.charIDToTypeID( "DChT" );
	desc4.putBoolean( id27, false );
	var id28 = app.charIDToTypeID( "DChV" );
	desc4.putBoolean( id28, false );
	var id29 = app.charIDToTypeID( "WebS" );
	desc4.putInteger( id29, 0 );
	var id30 = app.charIDToTypeID( "TDth" ); //transparency dither
	var id31 = app.charIDToTypeID( "IRDt" );
	var id32;
	switch (options.transparencyDither) {

	case Dither.NONE:
		id32 = app.charIDToTypeID( "None" );
		break;

	case Dither.DIFFUSION:
		id32 = app.charIDToTypeID( "Dfsn" );
		break;

	case Dither.PATTERN:
		id32 = app.charIDToTypeID( "Ptrn" );
		break;

	case Dither.NOISE:
		id32 = app.charIDToTypeID( "BNoi" );
		break;

	default:
		throw new Error("Unknown transparency dither algorithm. Cannot export PNG-8!");
	}
	desc4.putEnumerated( id30, id31, id32 );
	var id33 = app.charIDToTypeID( "TDtA" );
	desc4.putInteger( id33, options.transparencyAmount );
	var id34 = app.charIDToTypeID( "Trns" ); //Transparency
	desc4.putBoolean( id34, options.transparency );
	var id35 = app.charIDToTypeID( "Mtt " );
	desc4.putBoolean( id35, true );		 //matte
	var id36 = app.charIDToTypeID( "MttR" ); //matte color
	desc4.putInteger( id36, options.matteColor.red );
	var id37 = app.charIDToTypeID( "MttG" );
	desc4.putInteger( id37, options.matteColor.green );
	var id38 = app.charIDToTypeID( "MttB" );
	desc4.putInteger( id38, options.matteColor.blue );
	var id39 = app.charIDToTypeID( "SHTM" );
	desc4.putBoolean( id39, false );
	var id40 = app.charIDToTypeID( "SImg" );
	desc4.putBoolean( id40, true );
	var id41 = app.charIDToTypeID( "SSSO" );
	desc4.putBoolean( id41, false );
	var id42 = app.charIDToTypeID( "SSLt" );
	var list1 = new ActionList();
	desc4.putList( id42, list1 );
	var id43 = app.charIDToTypeID( "DIDr" );
	desc4.putBoolean( id43, false );
	var id44 = app.charIDToTypeID( "In  " );
	desc4.putPath( id44, new File(fileName) );
	var id45 = app.stringIDToTypeID( "SaveForWeb" );
	desc3.putObject( id6, id45, desc4 );
	app.executeAction( id5, desc3, DialogModes.NO );
}

//
// Utilities
//

function padder(input, padLength)
{
	var result = "Copy";//(new Array(padLength + 1 - input.toString().length)).join('0') + input;
	return result;
}

function makeValidFileName(fileName, replaceSpaces)
{
	//var validName = fileName.replace(/^\s+|\s+$/gm, '');	// trim spaces
	var validName = fileName.replace(/[\\\*\/\?:"\|<>&]/g, ''); // remove characters not allowed in a file name
	if (replaceSpaces) {
		validName = validName.replace(/[ ]/g, '_');			// replace spaces with underscores, since some programs still may have troubles with them
	}
	return validName;
}

function formatString(text)
{
	var args = Array.prototype.slice.call(arguments, 1);
	return text.replace(/\{(\d+)\}/g, function(match, number) {
		return (typeof args[number] != 'undefined') ? args[number] : match;
	});
}

function indexOf(array, element)
{
	var index = -1;
	for (var i = 0; i < array.length; ++i) {
		if (array[i] === element) {
			index = i;
			break;
		}
	}
	
	return index;
}

function loadResource(file)
{
	var rsrcString;
	if (! file.exists) {
		alert("Resource file '" + file.name + "' for the export dialog is missing! Please, download the rest of the files that come with this script.", "Error", true);
		return false;
	}
	try {
		file.open("r");
		if (file.error) throw file.error;
		rsrcString = file.read();
		if (file.error) throw file.error;
		if (! file.close()) {
			throw file.error;
		}
	}
	catch (error) {
		alert("Failed to read the resource file '" + file.name + "'!\n\nReason: " + error + "\n\nPlease, check it's available for reading and redownload it in case it became corrupted.", "Error", true);
		return false;
	}

	return rsrcString;
}

function Profiler(enabled)
{
	this.enabled = enabled;
	if (this.enabled) {
		this.startTime = new Date();
		this.lastTime = this.startTime;
	}
}


function getOS() {
	if ($.os.toLowerCase().search("macintosh") != -1)
		return 'macos';
	else if ($.os.toLowerCase().search("windows") != -1)
		return 'windows';
	else
		return 'unknown';
};

function getRemoveCommand() {
	if ('windows' == getOS())
		return 'del /f /q ';
	else if ('macos' == getOS())
		return 'rm -rf ';
}

function getCopyCommand() {
	if ('windows' == getOS())
		return 'xcopy /f /Y ';
	else if ('macos' == getOS())
		return 'cp -rf ';
}


function defineProfilerMethods()
{
	Profiler.prototype.getDuration = function(rememberAsLastCall, sinceLastCall)
	{
		if (this.enabled) {
			var currentTime = new Date();
			var lastTime = sinceLastCall ? this.lastTime : this.startTime;
			if (rememberAsLastCall) {
				this.lastTime = currentTime;
			}
			return new Date(currentTime.getTime() - lastTime.getTime());
		}
	}

	Profiler.prototype.resetLastTime = function()
	{
		this.lastTime = new Date();
	};

	Profiler.prototype.format = function(duration)
	{
		var output = padder(duration.getUTCHours(), 2) + ":";
		output += padder(duration.getUTCMinutes(), 2) + ":";
		output += padder(duration.getUTCSeconds(), 2) + ".";
		output += padder(duration.getUTCMilliseconds(), 3);
		return output;
	};
}

function undoChanges()
{
	var saveFile = File(savePSDPath);
	if(saveFile.exists)
	{
		 var name = exportPSDPath;
		 app.activeDocument.close(SaveOptions.DONOTSAVECHANGES);
		 var copyCmd = getCopyCommand() + "\"" +  File(name).fsName + "\" \"" + File(savePSDPath).fsName + "\"";

		if ('windows' == getOS())
		{
			executeSilently(copyCmd + "*");
		}
		else if ('macos' == getOS())
			app.system(copyCmd);
			
		 app.open(saveFile);
		
	}
	else
	{
		 var name = app.activeDocument.fullName;
		 app.activeDocument.close(SaveOptions.DONOTSAVECHANGES);
		 var copyCmd = getCopyCommand() + "\"" + File(exportPSDPath).fsName + "\" \"" + File(name).fsName + "\"";
		 
		if ('windows' == getOS())
		{
			executeSilently(copyCmd + "*");
		}
		else if ('macos' == getOS())
			app.system(copyCmd);

		var openFile = File(name);
		 app.open(openFile);
	 }
	 
	var command = getRemoveCommand() + "\"" + File(exportPSDPath).fsName + "\"";
	if ('windows' == getOS())
		executeSilently(command);
	else if ('macos' == getOS())
		app.system(command);
		
	alert("Exporting into SignStix Format Finished....");
}

undoChanges();

